package com.qmth.teachcloud.mark.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.teachcloud.mark.params.MarkObjectiveQuestionParams;
import com.qmth.teachcloud.mark.params.MarkQuestionParams;
import com.qmth.teachcloud.mark.entity.MarkPaper;
import com.qmth.teachcloud.mark.entity.MarkQuestion;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.enums.UploadFileEnum;
import com.qmth.teachcloud.mark.mapper.MarkQuestionMapper;
import com.qmth.teachcloud.mark.service.MarkPaperService;
import com.qmth.teachcloud.mark.service.MarkQuestionService;
import com.qmth.teachcloud.common.service.TeachcloudCommonService;
import com.qmth.teachcloud.common.util.FileStoreUtil;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 小题信息表 服务实现类
 * </p>
 *
 * @author xf
 * @since 2023-09-22
 */
@Service
public class MarkQuestionServiceImpl extends ServiceImpl<MarkQuestionMapper, MarkQuestion> implements MarkQuestionService {

    @Resource
    private TeachcloudCommonService teachcloudCommonService;

    @Resource
    private MarkPaperService markPaperService;

    @Resource
    private FileStoreUtil fileStoreUtil;

    @Override
    public String assembleGroupQuestionsByExamIdAndPaperNumberAndNumber(Long examId, String paperNumber, Integer groupNumber) {
        return this.baseMapper.assembleQuestionsByExamIdAndPaperNumberAndNumber(examId, paperNumber, groupNumber);
    }

    @Override
    public List<MarkQuestion> listQuestionByExamIdAndPaperNumber(Long examId, String paperNumber) {
        QueryWrapper<MarkQuestion> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(MarkQuestion::getExamId, examId)
                .eq(MarkQuestion::getPaperNumber, paperNumber)
                .orderByAsc(MarkQuestion::getMainNumber)
                .orderByAsc(MarkQuestion::getSubNumber);
        return this.list(queryWrapper);
    }

    @Transactional
    @Override
    public void saveQuestions(MarkQuestionParams markQuestionParams) {
        Long examId = markQuestionParams.getExamId();
        String paperNumber = markQuestionParams.getPaperNumber();
        List<MarkQuestion> questions = markQuestionParams.getQuestions();
        if (CollectionUtils.isEmpty(questions)) {
            throw ExceptionResultEnum.ERROR.exception("没有可保存的结构数据");
        }

        // 校验题号不能重复
        Map<String, Long> longMap = questions.stream().collect(Collectors.groupingBy(m -> String.valueOf(m.getMainNumber()) + m.getSubNumber(), Collectors.counting()));
        long count = longMap.entrySet().stream().filter(m -> m.getValue().intValue() > 1).count();
        if (count > 0) {
            throw ExceptionResultEnum.ERROR.exception("大题号+小题号有重复数据");
        }

        // 查询已有结构信息
        List<MarkQuestion> markQuestionList = this.listQuestionByExamIdAndPaperNumber(examId, paperNumber);
        if (CollectionUtils.isEmpty(markQuestionList)) {
            for (MarkQuestion question : questions) {
                if (question.getTotalScore() <= 0) {
                    throw ExceptionResultEnum.ERROR.exception("小题满分必须大于0");
                }
                if (question.getIntervalScore() <= 0) {
                    throw ExceptionResultEnum.ERROR.exception("间隔分必须大于0");
                }
                question.setExamId(examId);
                question.setPaperNumber(paperNumber);
            }
            this.saveBatch(questions);
        } else {
            List<MarkQuestion> saveOrUpdateList = new ArrayList<>();
            for (MarkQuestion question : questions) {
                if (Objects.isNull(question.getId())) {
                    saveOrUpdateList.add(question);
                } else {
                    Optional<MarkQuestion> questionOptional = markQuestionList.stream().filter(m -> m.getId().equals(question.getId())).findFirst();
                    if (questionOptional.isPresent()) {
                        MarkQuestion markQuestion = questionOptional.get();
                        if (markQuestion.getGroupNumber() != null && markQuestion.getGroupNumber().intValue() > 0 && markQuestion.getTotalScore().doubleValue() - question.getTotalScore().doubleValue() != 0) {
                            throw ExceptionResultEnum.ERROR.exception("大题号" + markQuestion.getMainNumber() + "小题号" + markQuestion.getSubNumber() + "已分组，不能修改小题满分");
                        }
                        if (question.getTotalScore() <= 0) {
                            throw ExceptionResultEnum.ERROR.exception("小题满分必须大于0");
                        }
                        if (question.getIntervalScore() <= 0) {
                            throw ExceptionResultEnum.ERROR.exception("间隔分必须大于0");
                        }
                        markQuestion.setTotalScore(question.getTotalScore());
                        saveOrUpdateList.add(markQuestion);
                        markQuestionList.remove(markQuestion);
                    } else {
                        throw ExceptionResultEnum.ERROR.exception("数据异常，请刷新页面后重新操作");
                    }
                }
            }
            // 删除的结构
            if (CollectionUtils.isNotEmpty(markQuestionList)) {
                List<Long> deleteIds = markQuestionList.stream().map(MarkQuestion::getId).collect(Collectors.toList());
                this.removeByIds(deleteIds);
            }
            this.saveOrUpdateBatch(saveOrUpdateList);
        }

    }

    @Override
    public List<MarkQuestion> listQuestionByExamIdAndPaperNumberAndGroupNumber(Long examId, String paperNumber, Integer number, Boolean isObjective) {
        QueryWrapper<MarkQuestion> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(MarkQuestion::getExamId, examId)
                .eq(MarkQuestion::getPaperNumber, paperNumber);
        if (number != null) {
            queryWrapper.lambda().eq(MarkQuestion::getGroupNumber, number);
        }
        if (isObjective != null) {
            queryWrapper.lambda().eq(MarkQuestion::getObjective, isObjective);
        }
        queryWrapper.lambda().orderByAsc(MarkQuestion::getMainNumber)
                .orderByAsc(MarkQuestion::getSubNumber);
        return this.list(queryWrapper);
    }

    @Transactional
    @Override
    public void saveObjectiveQuestions(MarkObjectiveQuestionParams markObjectiveQuestionParams) {
        for (MarkQuestion question : markObjectiveQuestionParams.getQuestions()) {
            if (StringUtils.isBlank(question.getAnswer())) {
                throw ExceptionResultEnum.ERROR.exception("答案不能为空");
            }
            UpdateWrapper<MarkQuestion> updateWrapper = new UpdateWrapper<>();
            updateWrapper.lambda().set(MarkQuestion::getAnswer, question.getAnswer());
            if (StringUtils.isNotBlank(question.getObjectivePolicy())) {
                updateWrapper.lambda().set(MarkQuestion::getObjectivePolicy, question.getObjectivePolicy());
            }
            updateWrapper.lambda().eq(MarkQuestion::getId, question.getId());
            this.update(updateWrapper);
        }
    }

    @Override
    public String uploadSubjectiveFile(Long examId, String paperNumber, MultipartFile file, String md5) {
        String answerFilePath = fileStoreUtil.uploadFile(file, md5, UploadFileEnum.PDF);
        UpdateWrapper<MarkPaper> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(MarkPaper::getAnswerFilePath, answerFilePath)
                .eq(MarkPaper::getExamId, examId)
                .eq(MarkPaper::getPaperNumber, paperNumber);
        markPaperService.update(updateWrapper);
        return teachcloudCommonService.filePreview(answerFilePath);
    }

    @Override
    public String previewAnswerFileByExamIdAndPaperNumber(Long examId, String paperNumber) {
        MarkPaper markPaper = markPaperService.getByExamIdAndPaperNumber(examId, paperNumber);
        if (markPaper != null) {
            return teachcloudCommonService.filePreview(markPaper.getAnswerFilePath());
        }
        return null;
    }

    @Override
    public double sumTotalScoreByGroupNumber(Long examId, String paperNumber, Integer groupNumber) {
        Double score = this.baseMapper.sumTotalScoreByGroupNumber(examId, paperNumber, groupNumber);
        return score != null ? score.doubleValue() : 0d;
    }

    @Override
    public long countQuestionByExamIdAndPaperNumberAndGroupNumberIsNull(Long examId, String paperNumber, boolean isObjective) {
        QueryWrapper<MarkQuestion> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(MarkQuestion::getExamId, examId)
                .eq(MarkQuestion::getPaperNumber, paperNumber)
                .eq(MarkQuestion::getObjective, isObjective)
                .isNull(MarkQuestion::getGroupNumber);
        return this.count(queryWrapper);
    }
}
