package com.qmth.teachcloud.mark.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.util.ExcelUtil;
import com.qmth.teachcloud.mark.dto.mark.manage.MarkManageDto;
import com.qmth.teachcloud.mark.dto.mark.manage.MarkerInfoDto;
import com.qmth.teachcloud.mark.entity.MarkGroup;
import com.qmth.teachcloud.mark.entity.MarkTask;
import com.qmth.teachcloud.mark.enums.MarkTaskStatus;
import com.qmth.teachcloud.mark.mapper.MarkTaskMapper;
import com.qmth.teachcloud.mark.service.MarkGroupService;
import com.qmth.teachcloud.mark.service.MarkQuestionService;
import com.qmth.teachcloud.mark.service.MarkTaskService;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.text.DecimalFormat;
import java.util.List;

/**
 * <p>
 * 评卷任务表 服务实现类
 * </p>
 *
 * @author xf
 * @since 2023-09-22
 */
@Service
public class MarkTaskServiceImpl extends ServiceImpl<MarkTaskMapper, MarkTask> implements MarkTaskService {

    @Resource
    private MarkGroupService markGroupService;

    @Resource
    private MarkQuestionService markQuestionService;

    @Override
    public IPage<MarkManageDto> listPaperManage(Long examId, String courseCode, String paperNumber, Boolean progressStatus, Integer pageNumber, Integer pageSize) {
        Page<MarkManageDto> page = new Page<>(pageNumber, pageSize);
        return this.baseMapper.listPaperManage(page, examId, courseCode, paperNumber, progressStatus);
    }

    @Override
    public void exportMarker(Long examId, String courseCode, String paperNumber, HttpServletResponse response) {
        List<MarkerInfoDto> markerInfoDtoList = this.baseMapper.listUserMarkedCount(examId, courseCode, paperNumber);
        for (MarkerInfoDto markerInfoDto : markerInfoDtoList) {
            MarkGroup markGroup = markGroupService.getByExamIdAndPaperNumberAndGroupNumber(examId, markerInfoDto.getPaperNumber(), markerInfoDto.getGroupNumber());
            markGroup.setQuestionList(markQuestionService.listQuestionByExamIdAndPaperNumberAndGroupNumber(examId, markerInfoDto.getPaperNumber(), markerInfoDto.getGroupNumber(), false));
            markerInfoDto.setGroupName(markGroup.getQuestionTitle());
            markerInfoDto.setTotalScore(markQuestionService.sumTotalScoreByGroupNumber(examId, markerInfoDto.getPaperNumber(), markerInfoDto.getGroupNumber()));
        }
        try {
            ExcelUtil.excelExport("评卷员工作量", MarkerInfoDto.class, markerInfoDtoList, response);
        } catch (Exception e) {
            throw ExceptionResultEnum.ERROR.exception("导出评卷员工作量失败");
        }
    }

    @Override
    public List<MarkTask> listByExamIdAndPaperNumberAndGroupNumberAndUserId(Long examId, String paperNumber, Integer groupNumber, Long userId) {
        QueryWrapper<MarkTask> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(MarkTask::getExamId, examId)
                .eq(MarkTask::getPaperNumber, paperNumber);
        if (groupNumber != null) {
            queryWrapper.lambda().eq(MarkTask::getGroupNumber, groupNumber);
        }
        if (userId != null) {
            queryWrapper.lambda().eq(MarkTask::getUserId, userId);
        }
        return this.list(queryWrapper);
    }

    @Override
    public Integer markedCount(List<MarkTask> markTask) {
        if (CollectionUtils.isEmpty(markTask)) {
            return 0;
        }
        long count = markTask.stream().filter(m -> m.getStatus().equals(MarkTaskStatus.MARKED) || m.getStatus().equals(MarkTaskStatus.ARBITRATED)).count();
        return (int) count;
    }

    @Override
    public Integer waitArbitrateCount(List<MarkTask> markTask) {
        if (CollectionUtils.isEmpty(markTask)) {
            return 0;
        }
        long count = markTask.stream().filter(m -> m.getStatus().equals(MarkTaskStatus.WAIT_ARBITRATE)).count();
        return (int) count;
    }

    @Override
    public Double calcPercent(Integer markedCount, Integer taskCount) {
        String percent = taskCount > 0 ? new DecimalFormat("####.###").format(markedCount * 100.0 / taskCount) : "0";
        return Double.parseDouble(percent);
    }

    @Override
    public int countByExamIdAndPaperNumberAndGroupNumberAndUserId(Long examId, String paperNumber, Integer groupNumber, Long userId) {
        QueryWrapper<MarkTask> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(MarkTask::getExamId, examId)
                .eq(MarkTask::getPaperNumber, paperNumber)
                .eq(MarkTask::getGroupNumber, groupNumber)
                .eq(MarkTask::getUserId, userId);
        return this.count(queryWrapper);
    }

    @Override
    public List<MarkTask> listByExamIdAndPaperNumberAndGroupNumberAndUserIdAndStatusNotIn(Long examId, String paperNumber, Integer groupNumber, Long userId, List<MarkTaskStatus> statusList) {
        QueryWrapper<MarkTask> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(MarkTask::getExamId, examId)
                .eq(MarkTask::getPaperNumber, paperNumber)
                .eq(MarkTask::getGroupNumber, groupNumber)
                .eq(MarkTask::getUserId, userId)
                .notIn(MarkTask::getStatus, statusList);
        return this.list(queryWrapper);
    }

    @Override
    public boolean resetById(Long markTaskId, Long userId, Long rejecterId, Long date, MarkTaskStatus newStatus) {
        UpdateWrapper<MarkTask> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(MarkTask::getStatus, newStatus)
                .set(MarkTask::getUserId, userId)
                .set(MarkTask::getMarkerTime, null)
                .set(MarkTask::getMarkerScore, null)
                .set(MarkTask::getMarkerScoreList, null)
                .set(MarkTask::getMarkerSpent, null)
                .set(MarkTask::getHeaderId, rejecterId)
                .set(MarkTask::getHeaderTime, date)
                .set(MarkTask::getHeaderScore, null)
                .set(MarkTask::getHeaderScoreList, null)
                .eq(MarkTask::getId, markTaskId);
        return this.update(updateWrapper);
    }

    @Override
    public int countByExamIdAndPaperNumberAndGroupNumberAndStatusIn(Long examId, String paperNumber, Integer groupNumber, List<MarkTaskStatus> statusList) {
        QueryWrapper<MarkTask> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(MarkTask::getExamId, examId)
                .eq(MarkTask::getPaperNumber, paperNumber)
                .eq(MarkTask::getGroupNumber, groupNumber)
                .in(MarkTask::getStatus, statusList);
        return this.count(queryWrapper);
    }

    @Override
    public MarkTask getLastOneByUserIdAndStatus(Long examId, String paperNumber, Integer groupNumber, Long userId, MarkTaskStatus status) {
        return this.baseMapper.getLastOneByUserIdAndStatus(examId, paperNumber, groupNumber, userId, status.name());
    }

    @Override
    public List<MarkTask> listByExamIdAndPaperNumberAndGroupNumberAndStudentId(Long examId, String paperNumber, Integer groupNumber, Long studentId) {
        QueryWrapper<MarkTask> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(MarkTask::getExamId, examId)
                .eq(MarkTask::getPaperNumber, paperNumber)
                .eq(MarkTask::getGroupNumber, groupNumber)
                .eq(MarkTask::getStudentId, studentId);
        return this.list(queryWrapper);
    }

    @Override
    public List<MarkTask> listByStudentIdAndGroupNumber(Long studentId, Integer groupNumber) {
        QueryWrapper<MarkTask> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(MarkTask::getStudentId, studentId)
                .eq(MarkTask::getGroupNumber, groupNumber);
        return this.list(queryWrapper);
    }

    @Override
    public void updateHeaderResult(Long examId, String paperNumber, Integer groupNumber, Long studentId, Long updateUserId, Double totalScore, String scoreList, Long updateTime, MarkTaskStatus arbitrated) {
        UpdateWrapper<MarkTask> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(MarkTask::getHeaderId, updateUserId)
                .set(MarkTask::getHeaderScore, totalScore)
                .set(MarkTask::getHeaderScoreList, scoreList)
                .set(MarkTask::getHeaderTime, updateTime)
                .set(MarkTask::getStatus, arbitrated)
                .eq(MarkTask::getExamId, examId)
                .eq(MarkTask::getPaperNumber, paperNumber)
                .eq(MarkTask::getGroupNumber, groupNumber)
                .eq(MarkTask::getStudentId, studentId);
        this.update(updateWrapper);
    }
}
